@@ -5,7 +5,7 @@ import logging
 import sys
 import typing
 from types import TracebackType
-from urllib.parse import SplitResult, parse_qsl, urlsplit
+from urllib.parse import SplitResult, parse_qsl, unquote, urlsplit
 
 from sqlalchemy import text
 from sqlalchemy.sql import ClauseElement
@@ -14,9 +14,9 @@ from databases.importer import import_from_string
 from databases.interfaces import ConnectionBackend, DatabaseBackend, TransactionBackend
 
 if sys.version_info >= (3, 7):  # pragma: no cover
-    from contextvars import ContextVar
+    import contextvars as contextvars
 else:  # pragma: no cover
-    from aiocontextvars import ContextVar
+    import aiocontextvars as contextvars
 
 try:  # pragma: no cover
     import click
@@ -62,13 +62,15 @@ class Database:
 
         self._force_rollback = force_rollback
 
-        backend_str = self.SUPPORTED_BACKENDS[self.url.scheme]
+        backend_str = self._get_backend()
         backend_cls = import_from_string(backend_str)
         assert issubclass(backend_cls, DatabaseBackend)
         self._backend = backend_cls(self.url, **self.options)
 
         # Connections are stored as task-local state.
-        self._connection_context = ContextVar("connection_context")  # type: ContextVar
+        self._connection_context = contextvars.ContextVar(
+            "connection_context"
+        )  # type: contextvars.ContextVar
 
         # When `force_rollback=True` is used, we use a single global
         # connection, within a transaction that always rolls back.
@@ -79,7 +81,9 @@ class Database:
         """
         Establish the connection pool.
         """
-        assert not self.is_connected, "Already connected."
+        if self.is_connected:
+            logger.debug("Already connected, skipping connection")
+            return None
 
         await self._backend.connect()
         logger.info(
@@ -102,7 +106,9 @@ class Database:
         """
         Close all connections in the connection pool.
         """
-        assert self.is_connected, "Already disconnected."
+        if not self.is_connected:
+            logger.debug("Already disconnected, skipping disconnection")
+            return None
 
         if self._force_rollback:
             assert self._global_connection is not None
@@ -112,6 +118,8 @@ class Database:
 
             self._global_transaction = None
             self._global_connection = None
+        else:
+            self._connection_context = contextvars.ContextVar("connection_context")
 
         await self._backend.disconnect()
         logger.info(
@@ -173,6 +181,11 @@ class Database:
             async for record in connection.iterate(query, values):
                 yield record
 
+    def _new_connection(self) -> "Connection":
+        connection = Connection(self._backend)
+        self._connection_context.set(connection)
+        return connection
+
     def connection(self) -> "Connection":
         if self._global_connection is not None:
             return self._global_connection
@@ -180,12 +193,23 @@ class Database:
         try:
             return self._connection_context.get()
         except LookupError:
-            connection = Connection(self._backend)
-            self._connection_context.set(connection)
-            return connection
+            return self._new_connection()
 
-    def transaction(self, *, force_rollback: bool = False) -> "Transaction":
-        return Transaction(self.connection, force_rollback=force_rollback)
+    def transaction(
+        self, *, force_rollback: bool = False, **kwargs: typing.Any
+    ) -> "Transaction":
+        try:
+            connection = self._connection_context.get()
+            is_root = not connection._transaction_stack
+            if is_root:
+                newcontext = contextvars.copy_context()
+                get_conn = lambda: newcontext.run(self._new_connection)
+            else:
+                get_conn = self.connection
+        except LookupError:
+            get_conn = self.connection
+
+        return Transaction(get_conn, force_rollback=force_rollback, **kwargs)
 
     @contextlib.contextmanager
     def force_rollback(self) -> typing.Iterator[None]:
@@ -196,6 +220,12 @@ class Database:
         finally:
             self._force_rollback = initial
 
+    def _get_backend(self) -> str:
+        try:
+            return self.SUPPORTED_BACKENDS[self.url.scheme]
+        except KeyError:
+            return self.SUPPORTED_BACKENDS[self.url.dialect]
+
 
 class Connection:
     def __init__(self, backend: DatabaseBackend) -> None:
@@ -213,8 +243,12 @@ class Connection:
     async def __aenter__(self) -> "Connection":
         async with self._connection_lock:
             self._connection_counter += 1
-            if self._connection_counter == 1:
-                await self._connection.acquire()
+            try:
+                if self._connection_counter == 1:
+                    await self._connection.acquire()
+            except Exception as e:
+                self._connection_counter -= 1
+                raise e
         return self
 
     async def __aexit__(
@@ -251,8 +285,7 @@ class Connection:
     ) -> typing.Any:
         built_query = self._build_query(query, values)
         async with self._query_lock:
-            row = await self._connection.fetch_one(built_query)
-        return None if row is None else row[column]
+            return await self._connection.fetch_val(built_query, column)
 
     async def execute(
         self, query: typing.Union[ClauseElement, str], values: dict = None
@@ -277,11 +310,13 @@ class Connection:
                 async for record in self._connection.iterate(built_query):
                     yield record
 
-    def transaction(self, *, force_rollback: bool = False) -> "Transaction":
+    def transaction(
+        self, *, force_rollback: bool = False, **kwargs: typing.Any
+    ) -> "Transaction":
         def connection_callable() -> Connection:
             return self
 
-        return Transaction(connection_callable, force_rollback)
+        return Transaction(connection_callable, force_rollback, **kwargs)
 
     @property
     def raw_connection(self) -> typing.Any:
@@ -306,9 +341,11 @@ class Transaction:
         self,
         connection_callable: typing.Callable[[], Connection],
         force_rollback: bool,
+        **kwargs: typing.Any,
     ) -> None:
         self._connection_callable = connection_callable
         self._force_rollback = force_rollback
+        self._extra_options = kwargs
 
     async def __aenter__(self) -> "Transaction":
         """
@@ -356,7 +393,9 @@ class Transaction:
         async with self._connection._transaction_lock:
             is_root = not self._connection._transaction_stack
             await self._connection.__aenter__()
-            await self._transaction.start(is_root=is_root)
+            await self._transaction.start(
+                is_root=is_root, extra_options=self._extra_options
+            )
             self._connection._transaction_stack.append(self)
         return self
 
@@ -382,7 +421,14 @@ class _EmptyNetloc(str):
 
 class DatabaseURL:
     def __init__(self, url: typing.Union[str, "DatabaseURL"]):
-        self._url = str(url)
+        if isinstance(url, DatabaseURL):
+            self._url: str = url._url
+        elif isinstance(url, str):
+            self._url = url
+        else:
+            raise TypeError(
+                f"Invalid type for DatabaseURL. Expected str or DatabaseURL, got {type(url)}"
+            )
 
     @property
     def components(self) -> SplitResult:
@@ -404,13 +450,26 @@ class DatabaseURL:
             return ""
         return self.components.scheme.split("+", 1)[1]
 
+    @property
+    def userinfo(self) -> typing.Optional[bytes]:
+        if self.components.username:
+            info = self.components.username
+            if self.components.password:
+                info += ":" + self.components.password
+            return info.encode("utf-8")
+        return None
+
     @property
     def username(self) -> typing.Optional[str]:
-        return self.components.username
+        if self.components.username is None:
+            return None
+        return unquote(self.components.username)
 
     @property
     def password(self) -> typing.Optional[str]:
-        return self.components.password
+        if self.components.password is None:
+            return None
+        return unquote(self.components.password)
 
     @property
     def hostname(self) -> typing.Optional[str]:
@@ -429,7 +488,7 @@ class DatabaseURL:
         path = self.components.path
         if path.startswith("/"):
             path = path[1:]
-        return path
+        return unquote(path)
 
     @property
     def options(self) -> dict:
@@ -446,8 +505,8 @@ class DatabaseURL:
         ):
             hostname = kwargs.pop("hostname", self.hostname)
             port = kwargs.pop("port", self.port)
-            username = kwargs.pop("username", self.username)
-            password = kwargs.pop("password", self.password)
+            username = kwargs.pop("username", self.components.username)
+            password = kwargs.pop("password", self.components.password)
 
             netloc = hostname
             if port is not None:
